//
// Base de travail
//
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JLabel;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import java.io.IOException;

//
// Pour les arbres
//
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;

//
// Pour XML
//
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

//
// Autres
//
import java.net.URL;
import javax.swing.JEditorPane;
import javax.swing.UIManager;
import java.beans.XMLDecoder;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;



public class VisualisationEnArbre
   extends JFrame
   implements TreeSelectionListener {
   private static final String
      TITRE_DFAUT = "Dmonstrateur de la visualisation en arbre";

   private JTree m_Arbre;
   private PanneauHTML m_PanneauHTML;
   private URL m_URLAide;

   public VisualisationEnArbre () {
      super (TITRE_DFAUT);
      m_Arbre = null;
      m_PanneauHTML = null;
      m_URLAide = null;
   }
   //
   //
   //
   private class paireNomURL {
      private String m_Nom;
      private URL m_URL;

      public URL getURL () {
         return m_URL;
      }
      private void setURL (URL url) {
         m_URL = url;
      }

      public String getNom () {
         return m_Nom;
      }
      private void setNom (String Nom) {
         m_Nom = Nom;
      }

      public paireNomURL(String Nom, String Fichier) {
         setNom (Nom);
         setURL (VisualisationEnArbre.class.getResource(Fichier));
         if (getURL () == null) {
             System.err.println("Ne peut trouver " + Fichier);
         }
      }

      public String toString() {
         return getNom ();
      }
   }

   private static String trouverNomHTML (String NomNoeud) {
      final String NomTest = NomNoeud + ".html";
      final String NOM_DFAUT= "Aide.html";
      File f = new File (NomTest);
      String Nom = (f.exists ())? NomTest : NOM_DFAUT;
      return Nom;
   }

   private void dcoder (Node n, DefaultMutableTreeNode dmtn) {
      if (n != null && n.toString ().trim ().length () > 0) {
         DefaultMutableTreeNode category = null;
         if (n instanceof Element) {
            Element elem = (Element) n;
            category = new DefaultMutableTreeNode(elem.getTagName ());
            dmtn.add(category);
         } else {
            dmtn.add(new DefaultMutableTreeNode(new paireNomURL (n.getNodeValue (), trouverNomHTML (n.getNodeValue ()))));
            category = dmtn;
         }
         if (n.hasAttributes ()) {
            NamedNodeMap nnm = n.getAttributes ();
            for (int i = 0; i < nnm.getLength (); i++) {
               category.add
                  (new DefaultMutableTreeNode
                     (new paireNomURL
                        (nnm.item (i).getNodeValue (), trouverNomHTML (n.getNodeValue ()))));
            }
         }
         if (n.getFirstChild () != null) {
            NodeList nl = n.getChildNodes ();
            dcoder (nl, category);
         }
      }
   }

   private void dcoder(NodeList nl, DefaultMutableTreeNode dmtn) {
      if (nl != null && nl.getLength () > 0) {
         for (int i = 0; i < nl.getLength (); i++) {
            Node n = nl.item (i);
            dcoder (n, dmtn);
         }
      }
   }

   private DefaultMutableTreeNode dcoder (String NomFich) {
      DefaultMutableTreeNode racine = null;
      try {
         DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
         DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
         Document doc = docBuilder.parse(new File (NomFich));
         Element elem = doc.getDocumentElement ();
         racine = new DefaultMutableTreeNode (elem.getTagName ());
         NodeList nl = elem.getChildNodes ();
         dcoder (nl, racine);
      } catch (ParserConfigurationException pce) {
         System.err.println (pce);
      } catch (SAXException saxe) {
         System.err.println (saxe);
      } catch (IOException ioe) {
         System.err.println (ioe);
      }
      return racine;
   }

   public void valueChanged(TreeSelectionEvent e) {
      DefaultMutableTreeNode Noeud =
         (DefaultMutableTreeNode) m_Arbre.getLastSelectedPathComponent();
      if (Noeud != null) {
         Object InfoNoeud = Noeud.getUserObject();
         if (Noeud.isLeaf()) {
            paireNomURL paire = (paireNomURL)InfoNoeud;
            m_PanneauHTML.afficherURL(paire.getURL ());
         } else {
            m_PanneauHTML.afficherURL(m_URLAide);
         }
      }
   }

   private class PanneauHTML extends JEditorPane {
      private static final String
         AIDE_DFAUT = "Aide.html";
      public PanneauHTML () {
         super ();
         setEditable (false);
         init ();
      }
      public void init () {
         final String s = AIDE_DFAUT ;
         m_URLAide = VisualisationEnArbre.class.getResource(s);
         if (m_URLAide == null) {
            System.err.println("Incapable d'ouvrir le fichier: " + s);
         }
         afficherURL(m_URLAide);
      }
      public void afficherURL(URL url)
      {
         try {
            if (url != null) {
               setPage(url);
            } else {
               setText("Fichier introuvable");
               System.out.println("Tentative d'afficher une URL nulle.");
            }
         } catch (IOException e) {
            System.err.println("Erreur  la lecture de l'URL " + url);
         }
      }
   }

   public class XMLFilter extends FileFilter
   {
      public boolean accept(File f) {
         final String EXT = ".xml";
         return (f.isDirectory() ||
                 f.getName ().substring (f.getName ().length () - EXT.length ()).equals (EXT));
      }
      public String getDescription() {
         return "XML seulement";
      }
   }

   private String SaisirFichier () {
      String fich = null;
      JFileChooser fc = new JFileChooser(".");
      fc.setFileFilter(new XMLFilter ());
      return (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)?
         fc.getSelectedFile().getAbsolutePath() : null;
   }

   public void init (String fich) {
      DefaultMutableTreeNode top = dcoder (fich);
      m_Arbre = new JTree(top);
      m_Arbre.getSelectionModel().setSelectionMode
         (TreeSelectionModel.SINGLE_TREE_SELECTION);
      m_Arbre.addTreeSelectionListener(this);

      m_PanneauHTML = new PanneauHTML ();

      JScrollPane VueHTML = new JScrollPane(m_PanneauHTML);
      JScrollPane VueArbre = new JScrollPane(m_Arbre);

      JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
      splitPane.setLeftComponent(VueArbre);
      splitPane.setRightComponent(VueHTML);

      Dimension minimumSize = new Dimension(100, 50);
      VueHTML.setMinimumSize(minimumSize);
      VueArbre.setMinimumSize(minimumSize);

      splitPane.setPreferredSize(new Dimension(500, 300));
      getContentPane ().add(splitPane);

      JButton Piton = new JButton ("xml?");
      TreeSelectionListener patch = this;
      Piton.addActionListener (
         new ActionListener () {
            public void actionPerformed (ActionEvent ae) {
               final String fich = SaisirFichier();
               if (fich != null) {
                  VisualisationEnArbre vea = new VisualisationEnArbre ();
                  vea.init (fich);
                  vea.pack ();
                  vea.setVisible (true);
               }
            }
         }
      );
      getContentPane ().add (Piton, BorderLayout.SOUTH);

      addWindowListener (
         new WindowAdapter () {
            public void windowClosing (WindowEvent we) {
               System.exit (0);
            }
         }
      );
   }

   private static void AppliquerApparenceLocale () {
      try {
         UIManager.setLookAndFeel
            (UIManager.getSystemLookAndFeelClassName());
      } catch (Exception e) {
         System.err.println("Incapable d'appliquer l'apparence locale...");
      }
   }

   public static void main (String [] args) {
      AppliquerApparenceLocale ();
      VisualisationEnArbre vea = new VisualisationEnArbre ();
      vea.init ("TestSOAPResponse.xml");
      vea.pack ();
      vea.setVisible (true);
   }
}
